""" 
Using the parseClass, build a topology that speficies a parser to parse
these sentences.  

turn step  Sentences
Turn left.
Step forward.
"""

import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle


from nealCoverClass import NealCoverFunctions
from parseClass import ParseClass

totalSentences = -1#set explicitly
lengthOfLongestSentence = -1 #set explicitly
neal = None

def addTurnSentences(parser):
    parser.addSentence(['turn','left','.'])
    turnState = parser.addState()
    parser.addTransition(parser.startState,turnState)
    parser.addTransitionOnWord('turn',turnState)
    turnLeftState = parser.addState()
    parser.addTransition(turnState,turnLeftState)
    parser.addTransitionOnWord('left',turnLeftState)
    tLPFinalState = parser.addState()
    parser.addTransition(turnLeftState,tLPFinalState)
    parser.addTransitionOnWord('.',tLPFinalState)
    return [tLPFinalState]

def linkInSink(spikeSource,sinkCell):
    global neal
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 


#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def allocateSinks(firstSink,spikeGenerators):
    if (nealParameters.simulator != 'spinnaker'):
        return
    sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    for sinkNumber  in range (firstSink,7):
        linkInSink(spikeGenerators[sinkNumber],sinkCell)

def setupTestSentences(parser,spikeGenerators,sentence):
    if (sentence == 0):
        parser.testTurnOnWord("turn",spikeGenerators[1])
        parser.testTurnOnWord("left",spikeGenerators[2])
        parser.testTurnOnWord(".",spikeGenerators[3])
        allocateSinks(4,spikeGenerators)
    elif (sentence == 1):
        parser.testTurnOnWord("step",spikeGenerators[1])
        parser.testTurnOnWord("forward",spikeGenerators[2])
        parser.testTurnOnWord(".",spikeGenerators[3])
        allocateSinks(4,spikeGenerators)


#access functions for variables from this file
def getTotalSentences():
    return totalSentences

def getLengthOfLongestSentence():
    return lengthOfLongestSentence

#---main 
def main():
    global neal
    global totalSentences
    global lengthOfLongestSentence

    #allocate neal and a parser
    neal = NealCoverFunctions(nealParameters.simulator)
    parser = ParseClass(nealParameters.simulator)

    #specify the sentences you want to parse.
    tFinalStates = addTurnSentences(parser)
    parser.addSentence(['step','forward','.']) 
    stepState = parser.addStateToNewStateOnWord(parser.startState,'step')
    stepForwardState = parser.addStateToNewStateOnWord(stepState,'forward')
    stepForwardFinalState = parser.addStateToNewStateOnWord(stepForwardState,'.')

    totalSentences = 2
    lengthOfLongestSentence = 2

    #optionally, print out information about the parser
    parser.printWords()
    #parser.printStateTransitions()
    #parser.printWordTransitions()

    print "Final states in order"
    print tFinalStates, stepForwardFinalState

    #allocate the topology
    parser.createNeurons()
    parser.recordNeurons()
    parser.createCAs()
    parser.createTransitionSynapses()

    #access the topology via the parser
    return parser



